
# This workflow is a practical GIS extension of the building classification method of Sturrock et al (2018). It allows output of model train/test res/non-res attributes, and model predictions and recoded (i.e. classified to res=1 or non-res=0) predictions, to csv format tables for visualisation with subsequent GDAL workflow

setwd("/mainfs/scratch/username/sturrockwindow/")

# Load workspace with Sturrock model output and packages
#load("/mainfs/scratch/username/sturrockwindow/rworkspaceCOD.RData")


#to display data clearly
#structure_points_testCODdf <- as.data.frame(structure_points_testCOD)
#sl_model_binomSL.predictdf <- as.data.frame(sl_model_binom$SL.predict)


# Prediction quantities from the superlearner model are returned in the form of probabilities. A cut off threshold is needed to determine binomial classification as a one or zero
# The threshold is determined in training with cross-validation
# Recode probabilities (using threshold as value obtained from trained model (opt_cutoff_SL))
# If p exceeds threshold, 1 else 0: res_or_nonres
# Use dplyr's ifelse() function to recode the probabilities to the threshold:

library(dplyr)

res_or_nonrespred <- ifelse(sl_model_binom$SL.predict>=opt_cutoff_SL$opt_cutoff,1,0)

CODpred1 <- as.data.frame(res_or_nonrespred)
CODpred2 <- as.data.frame(prediction_structure_pointsCOD)

library(data.table)
setDT(CODpred1, keep.rownames = "Row")
setDT(CODpred2, keep.rownames = "Row")

CODpred1$Row <- as.numeric(as.character(CODpred1$Row))
CODpred2$Row <- as.numeric(as.character(CODpred2$Row))

CODpred3 <- merge(CODpred1, CODpred2, by= "Row", all= TRUE)

# output dataframe to csv
write.csv(CODpred3, file = "/mainfs/scratch/username/sturrockwindow/CODpred3.csv", sep = ",", na = "", row.names = TRUE, col.names = TRUE)


## and output raw probabilities attached to polygon ids, in addition to the above recode derived from the training data
CODpred0<- as.data.frame(sl_model_binom$SL.predict)
setDT(CODpred0, keep.rownames = "Row")
setDT(CODpred2, keep.rownames = "Row")

CODpred0$Row <- as.numeric(as.character(CODpred0$Row))
CODpred2$Row <- as.numeric(as.character(CODpred2$Row))

CODpred4 <- merge(CODpred0, CODpred2, by= "Row", all= TRUE)

# output dataframe to csv
write.csv(CODpred4, file = "/mainfs/scratch/username/sturrockwindow/CODpred4.csv", sep = ",", na = "", row.names = TRUE, col.names = TRUE)


## output train/test res/non-res label attributes attached to polygon ids, in a similar fashion
CODtraintest <- as.data.frame(structure_pointsCOD)
setDT(CODtraintest, keep.rownames = "Row")
CODtraintest$Row <- as.numeric(as.character(CODtraintest$Row))

# output dataframe to csv
write.csv(CODtraintest, file = "/mainfs/scratch/username/sturrockwindow/CODtraintest.csv", sep = ",", na = "", row.names = TRUE, col.names = TRUE)
